# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.SerialKeyOptions import SerialKeyOptions
from .options.SerialKeyStatusIsActiveOptions import SerialKeyStatusIsActiveOptions
from .options.SerialKeyStatusIsUsedOptions import SerialKeyStatusIsUsedOptions
from .options.SerialKeyStatusIsInactiveOptions import SerialKeyStatusIsInactiveOptions
from .enum.SerialKeyStatus import SerialKeyStatus


class SerialKey:
    campaign_model_name: str
    code: str
    status: SerialKeyStatus
    created_at: int
    updated_at: int
    metadata: Optional[str] = None
    used_user_id: Optional[str] = None
    used_at: Optional[int] = None

    def __init__(
        self,
        campaign_model_name: str,
        code: str,
        status: SerialKeyStatus,
        created_at: int,
        updated_at: int,
        options: Optional[SerialKeyOptions] = SerialKeyOptions(),
    ):
        self.campaign_model_name = campaign_model_name
        self.code = code
        self.status = status
        self.created_at = created_at
        self.updated_at = updated_at
        self.metadata = options.metadata if options.metadata else None
        self.used_user_id = options.used_user_id if options.used_user_id else None
        self.used_at = options.used_at if options.used_at else None

    @staticmethod
    def status_is_active(
        campaign_model_name: str,
        code: str,
        created_at: int,
        updated_at: int,
        options: Optional[SerialKeyStatusIsActiveOptions] = SerialKeyStatusIsActiveOptions(),
    ) -> SerialKey:
        return SerialKey(
            campaign_model_name,
            code,
            SerialKeyStatus.ACTIVE,
            created_at,
            updated_at,
            SerialKeyOptions(
                options.metadata,
                options.used_at,
            ),
        )

    @staticmethod
    def status_is_used(
        campaign_model_name: str,
        code: str,
        created_at: int,
        updated_at: int,
        used_user_id: str,
        options: Optional[SerialKeyStatusIsUsedOptions] = SerialKeyStatusIsUsedOptions(),
    ) -> SerialKey:
        return SerialKey(
            campaign_model_name,
            code,
            SerialKeyStatus.USED,
            created_at,
            updated_at,
            SerialKeyOptions(
                used_user_id,
                options.metadata,
                options.used_at,
            ),
        )

    @staticmethod
    def status_is_inactive(
        campaign_model_name: str,
        code: str,
        created_at: int,
        updated_at: int,
        options: Optional[SerialKeyStatusIsInactiveOptions] = SerialKeyStatusIsInactiveOptions(),
    ) -> SerialKey:
        return SerialKey(
            campaign_model_name,
            code,
            SerialKeyStatus.INACTIVE,
            created_at,
            updated_at,
            SerialKeyOptions(
                options.metadata,
                options.used_at,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.campaign_model_name is not None:
            properties["campaignModelName"] = self.campaign_model_name
        if self.code is not None:
            properties["code"] = self.code
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.status is not None:
            properties["status"] = self.status.value
        if self.used_user_id is not None:
            properties["usedUserId"] = self.used_user_id
        if self.created_at is not None:
            properties["createdAt"] = self.created_at
        if self.used_at is not None:
            properties["usedAt"] = self.used_at
        if self.updated_at is not None:
            properties["updatedAt"] = self.updated_at

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .BalanceParameterModel import BalanceParameterModel
from .options.BalanceParameterModelOptions import BalanceParameterModelOptions
from .enum.BalanceParameterModelInitialValueStrategy import BalanceParameterModelInitialValueStrategy
from .RarityParameterModel import RarityParameterModel
from .options.RarityParameterModelOptions import RarityParameterModelOptions
from .BalanceParameterValueModel import BalanceParameterValueModel
from .options.BalanceParameterValueModelOptions import BalanceParameterValueModelOptions
from .RarityParameterCountModel import RarityParameterCountModel
from .options.RarityParameterCountModelOptions import RarityParameterCountModelOptions
from .RarityParameterValueModel import RarityParameterValueModel
from .options.RarityParameterValueModelOptions import RarityParameterValueModelOptions
from .BalanceParameterValue import BalanceParameterValue
from .options.BalanceParameterValueOptions import BalanceParameterValueOptions
from .RarityParameterValue import RarityParameterValue
from .options.RarityParameterValueOptions import RarityParameterValueOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr

from ..ref.RoomRef import RoomRef

from .options.RoomOptions import RoomOptions


class Room(CdkResource):
    stack: Stack
    owner_id: str
    namespace_name: str
    name: str
    ip_address: Optional[str] = None
    port: Optional[int] = None
    encryption_key: Optional[str] = None

    def __init__(
        self,
        stack: Stack,
        owner_id: str,
        namespace_name: str,
        name: str,
        options: Optional[RoomOptions] = RoomOptions(),
    ):
        super().__init__(
            "Realtime_Room_" + name
        )

        self.stack = stack
        self.owner_id = owner_id
        self.namespace_name = namespace_name
        self.name = name
        self.ip_address = options.ip_address if options.ip_address else None
        self.port = options.port if options.port else None
        self.encryption_key = options.encryption_key if options.encryption_key else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Realtime::Room"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.owner_id is not None:
            properties["OwnerId"] = self.owner_id
        if self.namespace_name is not None:
            properties["NamespaceName"] = self.namespace_name
        if self.name is not None:
            properties["Name"] = self.name
        if self.ip_address is not None:
            properties["IpAddress"] = self.ip_address
        if self.port is not None:
            properties["Port"] = self.port
        if self.encryption_key is not None:
            properties["EncryptionKey"] = self.encryption_key

        return properties

    def ref(
        self,
    ) -> RoomRef:
        return RoomRef(
            self.namespace_name,
            self.name,
        )

    def get_attr_room_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.RoomId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .options.NodeModelOptions import NodeModelOptions


class NodeModel:
    name: str
    release_consume_actions: List[ConsumeAction]
    restrain_return_rate: float
    metadata: Optional[str] = None
    return_acquire_actions: Optional[List[AcquireAction]] = None
    premise_node_names: Optional[List[str]] = None

    def __init__(
        self,
        name: str,
        release_consume_actions: List[ConsumeAction],
        restrain_return_rate: float,
        options: Optional[NodeModelOptions] = NodeModelOptions(),
    ):
        self.name = name
        self.release_consume_actions = release_consume_actions
        self.restrain_return_rate = restrain_return_rate
        self.metadata = options.metadata if options.metadata else None
        self.return_acquire_actions = options.return_acquire_actions if options.return_acquire_actions else None
        self.premise_node_names = options.premise_node_names if options.premise_node_names else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.release_consume_actions is not None:
            properties["releaseConsumeActions"] = [
                v.properties(
                )
                for v in self.release_consume_actions
            ]
        if self.restrain_return_rate is not None:
            properties["restrainReturnRate"] = self.restrain_return_rate
        if self.premise_node_names is not None:
            properties["premiseNodeNames"] = self.premise_node_names

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .NamespaceRef import NamespaceRef
from .RateModelRef import RateModelRef


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .VersionModel import VersionModel
from .options.VersionModelOptions import VersionModelOptions
from .enum.VersionModelScope import VersionModelScope
from .enum.VersionModelType import VersionModelType
from .options.VersionModelTypeIsSimpleOptions import VersionModelTypeIsSimpleOptions
from .options.VersionModelTypeIsScheduleOptions import VersionModelTypeIsScheduleOptions
from .options.VersionModelScopeIsPassiveOptions import VersionModelScopeIsPassiveOptions
from .options.VersionModelScopeIsActiveOptions import VersionModelScopeIsActiveOptions
from .Status import Status
from .options.StatusOptions import StatusOptions
from .TargetVersion import TargetVersion
from .options.TargetVersionOptions import TargetVersionOptions
from .SignTargetVersion import SignTargetVersion
from .options.SignTargetVersionOptions import SignTargetVersionOptions
from .Version import Version
from .options.VersionOptions import VersionOptions
from .ScheduleVersion import ScheduleVersion
from .options.ScheduleVersionOptions import ScheduleVersionOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from ..stamp_sheet.AcquireBigItemByUserId import AcquireBigItemByUserId
from ..stamp_sheet.SetBigItemByUserId import SetBigItemByUserId
from ..stamp_sheet.ConsumeBigItemByUserId import ConsumeBigItemByUserId
from ..stamp_sheet.VerifyBigItemByUserId import VerifyBigItemByUserId


class BigItemModelRef:
    namespace_name: str
    inventory_name: str
    item_name: str

    def __init__(
        self,
        namespace_name: str,
        inventory_name: str,
        item_name: str,
    ):
        self.namespace_name = namespace_name
        self.inventory_name = inventory_name
        self.item_name = item_name

    def acquire_big_item(
        self,
        acquire_count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireBigItemByUserId:
        return AcquireBigItemByUserId(
            self.namespace_name,
            self.inventory_name,
            self.item_name,
            acquire_count,
            user_id,
        )

    def set_big_item(
        self,
        count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> SetBigItemByUserId:
        return SetBigItemByUserId(
            self.namespace_name,
            self.inventory_name,
            self.item_name,
            count,
            user_id,
        )

    def consume_big_item(
        self,
        consume_count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeBigItemByUserId:
        return ConsumeBigItemByUserId(
            self.namespace_name,
            self.inventory_name,
            self.item_name,
            consume_count,
            user_id,
        )

    def verify_big_item(
        self,
        verify_type: str,
        count: str,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyBigItemByUserId:
        return VerifyBigItemByUserId(
            self.namespace_name,
            self.inventory_name,
            self.item_name,
            verify_type,
            count,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "inventory",
                self.namespace_name,
                "big",
                "model",
                self.inventory_name,
                "item",
                self.item_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .MissionTaskModel import MissionTaskModel
from .options.MissionGroupModelOptions import MissionGroupModelOptions
from .options.MissionGroupModelResetTypeIsNotResetOptions import MissionGroupModelResetTypeIsNotResetOptions
from .options.MissionGroupModelResetTypeIsDailyOptions import MissionGroupModelResetTypeIsDailyOptions
from .options.MissionGroupModelResetTypeIsWeeklyOptions import MissionGroupModelResetTypeIsWeeklyOptions
from .options.MissionGroupModelResetTypeIsMonthlyOptions import MissionGroupModelResetTypeIsMonthlyOptions
from .enum.MissionGroupModelResetType import MissionGroupModelResetType
from .enum.MissionGroupModelResetDayOfWeek import MissionGroupModelResetDayOfWeek


class MissionGroupModel:
    name: str
    reset_type: MissionGroupModelResetType
    metadata: Optional[str] = None
    tasks: Optional[List[MissionTaskModel]] = None
    reset_day_of_month: Optional[int] = None
    reset_day_of_week: Optional[MissionGroupModelResetDayOfWeek] = None
    reset_hour: Optional[int] = None
    complete_notification_namespace_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        reset_type: MissionGroupModelResetType,
        options: Optional[MissionGroupModelOptions] = MissionGroupModelOptions(),
    ):
        self.name = name
        self.reset_type = reset_type
        self.metadata = options.metadata if options.metadata else None
        self.tasks = options.tasks if options.tasks else None
        self.reset_day_of_month = options.reset_day_of_month if options.reset_day_of_month else None
        self.reset_day_of_week = options.reset_day_of_week if options.reset_day_of_week else None
        self.reset_hour = options.reset_hour if options.reset_hour else None
        self.complete_notification_namespace_id = options.complete_notification_namespace_id if options.complete_notification_namespace_id else None

    @staticmethod
    def reset_type_is_not_reset(
        name: str,
        options: Optional[MissionGroupModelResetTypeIsNotResetOptions] = MissionGroupModelResetTypeIsNotResetOptions(),
    ) -> MissionGroupModel:
        return MissionGroupModel(
            name,
            MissionGroupModelResetType.NOT_RESET,
            MissionGroupModelOptions(
                options.metadata,
                options.tasks,
                options.complete_notification_namespace_id,
            ),
        )

    @staticmethod
    def reset_type_is_daily(
        name: str,
        reset_hour: int,
        options: Optional[MissionGroupModelResetTypeIsDailyOptions] = MissionGroupModelResetTypeIsDailyOptions(),
    ) -> MissionGroupModel:
        return MissionGroupModel(
            name,
            MissionGroupModelResetType.DAILY,
            MissionGroupModelOptions(
                reset_hour,
                options.metadata,
                options.tasks,
                options.complete_notification_namespace_id,
            ),
        )

    @staticmethod
    def reset_type_is_weekly(
        name: str,
        reset_day_of_week: MissionGroupModelResetDayOfWeek,
        reset_hour: int,
        options: Optional[MissionGroupModelResetTypeIsWeeklyOptions] = MissionGroupModelResetTypeIsWeeklyOptions(),
    ) -> MissionGroupModel:
        return MissionGroupModel(
            name,
            MissionGroupModelResetType.WEEKLY,
            MissionGroupModelOptions(
                reset_day_of_week,
                reset_hour,
                options.metadata,
                options.tasks,
                options.complete_notification_namespace_id,
            ),
        )

    @staticmethod
    def reset_type_is_monthly(
        name: str,
        reset_day_of_month: int,
        reset_hour: int,
        options: Optional[MissionGroupModelResetTypeIsMonthlyOptions] = MissionGroupModelResetTypeIsMonthlyOptions(),
    ) -> MissionGroupModel:
        return MissionGroupModel(
            name,
            MissionGroupModelResetType.MONTHLY,
            MissionGroupModelOptions(
                reset_day_of_month,
                reset_hour,
                options.metadata,
                options.tasks,
                options.complete_notification_namespace_id,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.tasks is not None:
            properties["tasks"] = [
                v.properties(
                )
                for v in self.tasks
            ]
        if self.reset_type is not None:
            properties["resetType"] = self.reset_type.value
        if self.reset_day_of_month is not None:
            properties["resetDayOfMonth"] = self.reset_day_of_month
        if self.reset_day_of_week is not None:
            properties["resetDayOfWeek"] = self.reset_day_of_week.value
        if self.reset_hour is not None:
            properties["resetHour"] = self.reset_hour
        if self.complete_notification_namespace_id is not None:
            properties["completeNotificationNamespaceId"] = self.complete_notification_namespace_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import NotificationSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    follow_script: Optional[ScriptSetting] = None
    unfollow_script: Optional[ScriptSetting] = None
    send_request_script: Optional[ScriptSetting] = None
    cancel_request_script: Optional[ScriptSetting] = None
    accept_request_script: Optional[ScriptSetting] = None
    reject_request_script: Optional[ScriptSetting] = None
    delete_friend_script: Optional[ScriptSetting] = None
    update_profile_script: Optional[ScriptSetting] = None
    follow_notification: Optional[NotificationSetting] = None
    receive_request_notification: Optional[NotificationSetting] = None
    accept_request_notification: Optional[NotificationSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Friend_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.follow_script = options.follow_script if options.follow_script else None
        self.unfollow_script = options.unfollow_script if options.unfollow_script else None
        self.send_request_script = options.send_request_script if options.send_request_script else None
        self.cancel_request_script = options.cancel_request_script if options.cancel_request_script else None
        self.accept_request_script = options.accept_request_script if options.accept_request_script else None
        self.reject_request_script = options.reject_request_script if options.reject_request_script else None
        self.delete_friend_script = options.delete_friend_script if options.delete_friend_script else None
        self.update_profile_script = options.update_profile_script if options.update_profile_script else None
        self.follow_notification = options.follow_notification if options.follow_notification else None
        self.receive_request_notification = options.receive_request_notification if options.receive_request_notification else None
        self.accept_request_notification = options.accept_request_notification if options.accept_request_notification else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Friend::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.follow_script is not None:
            properties["FollowScript"] = self.follow_script.properties(
            )
        if self.unfollow_script is not None:
            properties["UnfollowScript"] = self.unfollow_script.properties(
            )
        if self.send_request_script is not None:
            properties["SendRequestScript"] = self.send_request_script.properties(
            )
        if self.cancel_request_script is not None:
            properties["CancelRequestScript"] = self.cancel_request_script.properties(
            )
        if self.accept_request_script is not None:
            properties["AcceptRequestScript"] = self.accept_request_script.properties(
            )
        if self.reject_request_script is not None:
            properties["RejectRequestScript"] = self.reject_request_script.properties(
            )
        if self.delete_friend_script is not None:
            properties["DeleteFriendScript"] = self.delete_friend_script.properties(
            )
        if self.update_profile_script is not None:
            properties["UpdateProfileScript"] = self.update_profile_script.properties(
            )
        if self.follow_notification is not None:
            properties["FollowNotification"] = self.follow_notification.properties(
            )
        if self.receive_request_notification is not None:
            properties["ReceiveRequestNotification"] = self.receive_request_notification.properties(
            )
        if self.accept_request_notification is not None:
            properties["AcceptRequestNotification"] = self.accept_request_notification.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import TransactionSetting
from ...core.model import ScriptSetting
from ...core.model import NotificationSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .CurrentMasterData import CurrentMasterData
from .GlobalMessage import GlobalMessage

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    is_automatic_deleting_enabled: Optional[bool] = None
    transaction_setting: Optional[TransactionSetting] = None
    receive_message_script: Optional[ScriptSetting] = None
    read_message_script: Optional[ScriptSetting] = None
    delete_message_script: Optional[ScriptSetting] = None
    receive_notification: Optional[NotificationSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Inbox_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.is_automatic_deleting_enabled = options.is_automatic_deleting_enabled if options.is_automatic_deleting_enabled else None
        self.transaction_setting = options.transaction_setting if options.transaction_setting else None
        self.receive_message_script = options.receive_message_script if options.receive_message_script else None
        self.read_message_script = options.read_message_script if options.read_message_script else None
        self.delete_message_script = options.delete_message_script if options.delete_message_script else None
        self.receive_notification = options.receive_notification if options.receive_notification else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Inbox::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.is_automatic_deleting_enabled is not None:
            properties["IsAutomaticDeletingEnabled"] = self.is_automatic_deleting_enabled
        if self.transaction_setting is not None:
            properties["TransactionSetting"] = self.transaction_setting.properties(
            )
        if self.receive_message_script is not None:
            properties["ReceiveMessageScript"] = self.receive_message_script.properties(
            )
        if self.read_message_script is not None:
            properties["ReadMessageScript"] = self.read_message_script.properties(
            )
        if self.delete_message_script is not None:
            properties["DeleteMessageScript"] = self.delete_message_script.properties(
            )
        if self.receive_notification is not None:
            properties["ReceiveNotification"] = self.receive_notification.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )

    def master_data(
        self,
        global_messages: List[GlobalMessage],
    ) -> Namespace:
        CurrentMasterData(
            self.stack,
            self.name,
            global_messages,
        ).add_depends_on(
            self,
        )
        return self



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Threshold import Threshold
from .AcquireActionRate import AcquireActionRate
from .options.ExperienceModelOptions import ExperienceModelOptions


class ExperienceModel:
    name: str
    default_experience: int
    default_rank_cap: int
    max_rank_cap: int
    rank_threshold: Threshold
    metadata: Optional[str] = None
    acquire_action_rates: Optional[List[AcquireActionRate]] = None

    def __init__(
        self,
        name: str,
        default_experience: int,
        default_rank_cap: int,
        max_rank_cap: int,
        rank_threshold: Threshold,
        options: Optional[ExperienceModelOptions] = ExperienceModelOptions(),
    ):
        self.name = name
        self.default_experience = default_experience
        self.default_rank_cap = default_rank_cap
        self.max_rank_cap = max_rank_cap
        self.rank_threshold = rank_threshold
        self.metadata = options.metadata if options.metadata else None
        self.acquire_action_rates = options.acquire_action_rates if options.acquire_action_rates else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.default_experience is not None:
            properties["defaultExperience"] = self.default_experience
        if self.default_rank_cap is not None:
            properties["defaultRankCap"] = self.default_rank_cap
        if self.max_rank_cap is not None:
            properties["maxRankCap"] = self.max_rank_cap
        if self.rank_threshold is not None:
            properties["rankThreshold"] = self.rank_threshold.properties(
            )
        if self.acquire_action_rates is not None:
            properties["acquireActionRates"] = [
                v.properties(
                )
                for v in self.acquire_action_rates
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .InventoryModel import InventoryModel
from .options.InventoryModelOptions import InventoryModelOptions
from .ItemModel import ItemModel
from .options.ItemModelOptions import ItemModelOptions
from .SimpleInventoryModel import SimpleInventoryModel
from .options.SimpleInventoryModelOptions import SimpleInventoryModelOptions
from .SimpleItemModel import SimpleItemModel
from .options.SimpleItemModelOptions import SimpleItemModelOptions
from .BigInventoryModel import BigInventoryModel
from .options.BigInventoryModelOptions import BigInventoryModelOptions
from .BigItemModel import BigItemModel
from .options.BigItemModelOptions import BigItemModelOptions
from .AcquireCount import AcquireCount
from .options.AcquireCountOptions import AcquireCountOptions
from .ConsumeCount import ConsumeCount
from .options.ConsumeCountOptions import ConsumeCountOptions
from .HeldCount import HeldCount
from .options.HeldCountOptions import HeldCountOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr

from ..ref.NamespaceRef import NamespaceRef
from .enum.NamespaceType import NamespaceType

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    type: Optional[NamespaceType] = None
    gcp_credential_json: Optional[str] = None
    big_query_dataset_name: Optional[str] = None
    log_expire_days: Optional[int] = None
    aws_region: Optional[str] = None
    aws_access_key_id: Optional[str] = None
    aws_secret_access_key: Optional[str] = None
    firehose_stream_name: Optional[str] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Log_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.type = options.type if options.type else None
        self.gcp_credential_json = options.gcp_credential_json if options.gcp_credential_json else None
        self.big_query_dataset_name = options.big_query_dataset_name if options.big_query_dataset_name else None
        self.log_expire_days = options.log_expire_days if options.log_expire_days else None
        self.aws_region = options.aws_region if options.aws_region else None
        self.aws_access_key_id = options.aws_access_key_id if options.aws_access_key_id else None
        self.aws_secret_access_key = options.aws_secret_access_key if options.aws_secret_access_key else None
        self.firehose_stream_name = options.firehose_stream_name if options.firehose_stream_name else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Log::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.type is not None:
            properties["Type"] = self.type
        if self.gcp_credential_json is not None:
            properties["GcpCredentialJson"] = self.gcp_credential_json
        if self.big_query_dataset_name is not None:
            properties["BigQueryDatasetName"] = self.big_query_dataset_name
        if self.log_expire_days is not None:
            properties["LogExpireDays"] = self.log_expire_days
        if self.aws_region is not None:
            properties["AwsRegion"] = self.aws_region
        if self.aws_access_key_id is not None:
            properties["AwsAccessKeyId"] = self.aws_access_key_id
        if self.aws_secret_access_key is not None:
            properties["AwsSecretAccessKey"] = self.aws_secret_access_key
        if self.firehose_stream_name is not None:
            properties["FirehoseStreamName"] = self.firehose_stream_name

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import ConsumeAction
from ...core.model import AcquireAction
from .RandomDisplayItemModel import RandomDisplayItemModel
from .options.RandomShowcaseOptions import RandomShowcaseOptions


class RandomShowcase:
    name: str
    maximum_number_of_choice: int
    display_items: List[RandomDisplayItemModel]
    base_timestamp: int
    reset_interval_hours: int
    metadata: Optional[str] = None
    sales_period_event_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        maximum_number_of_choice: int,
        display_items: List[RandomDisplayItemModel],
        base_timestamp: int,
        reset_interval_hours: int,
        options: Optional[RandomShowcaseOptions] = RandomShowcaseOptions(),
    ):
        self.name = name
        self.maximum_number_of_choice = maximum_number_of_choice
        self.display_items = display_items
        self.base_timestamp = base_timestamp
        self.reset_interval_hours = reset_interval_hours
        self.metadata = options.metadata if options.metadata else None
        self.sales_period_event_id = options.sales_period_event_id if options.sales_period_event_id else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.maximum_number_of_choice is not None:
            properties["maximumNumberOfChoice"] = self.maximum_number_of_choice
        if self.display_items is not None:
            properties["displayItems"] = [
                v.properties(
                )
                for v in self.display_items
            ]
        if self.base_timestamp is not None:
            properties["baseTimestamp"] = self.base_timestamp
        if self.reset_interval_hours is not None:
            properties["resetIntervalHours"] = self.reset_interval_hours
        if self.sales_period_event_id is not None:
            properties["salesPeriodEventId"] = self.sales_period_event_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite
from .User import User
from .options.UserOptions import UserOptions
from .SecurityPolicy import SecurityPolicy
from .options.SecurityPolicyOptions import SecurityPolicyOptions
from .Identifier import Identifier
from .options.IdentifierOptions import IdentifierOptions
from .Password import Password
from .options.PasswordOptions import PasswordOptions
from .AttachSecurityPolicy import AttachSecurityPolicy
from .options.AttachSecurityPolicyOptions import AttachSecurityPolicyOptions
from .ProjectToken import ProjectToken
from .options.ProjectTokenOptions import ProjectTokenOptions
from .Policy import Policy
from .Statement import Statement